@@ -558,7 +558,7 @@ get_folder_online (MailStubExchangeFolder *mfld, gboolean background)
 	MailStub *stub = MAIL_STUB (mfld->mse);
 	E2kHTTPStatus status;
 	E2kResult *results;
-	int nresults;
+	int nresults = 0;
 	gboolean readonly;
 	E2kRestriction *rn;
 	E2kResultIter *iter;
@@ -599,6 +599,8 @@ get_folder_online (MailStubExchangeFolder *mfld, gboolean background)
 		if (!background) {
 			got_folder_error (mfld, _("Could not open folder: Permission denied"));
 		}
+		if (nresults)
+			e2k_results_free (results, nresults);
 		return FALSE;
 	}
 	readonly = (mfld->access & (MAPI_ACCESS_MODIFY | MAPI_ACCESS_CREATE_CONTENTS)) == 0;
@@ -704,6 +706,9 @@ get_folder_online (MailStubExchangeFolder *mfld, gboolean background)
 		if (!background) {
 			got_folder_error (mfld, _("Could not open folder"));
 		}
+		if (nresults)
+			e2k_results_free (results, nresults);
+
 		return FALSE;
 	}
 
@@ -725,6 +730,8 @@ get_folder_online (MailStubExchangeFolder *mfld, gboolean background)
 	if (background) {
 		mail_stub_push_changes (stub);
 	}
+	if (nresults)
+		e2k_results_free (results, nresults);
 	return TRUE;
 }
 
@@ -873,7 +880,7 @@ sync_deletions (MailStubExchange *mse, MailStubExchangeFolder *mfld)
 	MailStub *stub = MAIL_STUB (mse);
 	E2kHTTPStatus status;
 	E2kResult *results;
-	int nresults;
+	int nresults = 0;
 	const char *prop;
 	int deleted_count = -1, new_deleted_count, visible_count = -1, mode;
 	E2kRestriction *rn;
@@ -907,6 +914,8 @@ sync_deletions (MailStubExchange *mse, MailStubExchangeFolder *mfld)
 	if (prop)
 		visible_count = atoi (prop);
 
+	e2k_results_free (results, nresults);
+
 	if (visible_count >= mfld->messages->len) {
 		if (mfld->deleted_count == deleted_count)
 			return;
@@ -1858,7 +1867,7 @@ get_stickynote (E2kContext *ctx, E2kOperation *op, const char *uri,
 {
 	E2kHTTPStatus status;
 	E2kResult *results;
-	int nresults;
+	int nresults = 0;
 	GString *message;
 
 	status = e2k_context_propfind (ctx, op, uri,
@@ -1883,7 +1892,7 @@ build_message_from_document (E2kContext *ctx, E2kOperation *op,
 {
 	E2kHTTPStatus status;
 	E2kResult *results;
-	int nresults;
+	int nresults = 0;
 	GByteArray *message;
 	char *headers;
 
@@ -1907,6 +1916,7 @@ build_message_from_document (E2kContext *ctx, E2kOperation *op,
 	*body = (char *)message->data;
 	*len = message->len;
 	g_byte_array_free (message, FALSE);
+	e2k_results_free (results, nresults);
 	return status;
 }
 
@@ -1925,7 +1935,7 @@ unmangle_delegated_meeting_request (MailStubExchange *mse, E2kOperation *op,
 	EFolder *folder;
 	E2kHTTPStatus status;
 	E2kResult *results;
-	int nresults;
+	int nresults = 0;
 
 	status = e2k_context_propfind (mse->ctx, op, uri, &prop, 1,
 				       &results, &nresults);
@@ -1935,13 +1945,16 @@ unmangle_delegated_meeting_request (MailStubExchange *mse, E2kOperation *op,
 		return E2K_HTTP_MALFORMED;
 
 	delegator_dn = e2k_properties_get_prop (results[0].props, PR_RCVD_REPRESENTING_EMAIL_ADDRESS);
-	if (!delegator_dn)
+	if (!delegator_dn) {
+		e2k_results_free (results, nresults);
 		return E2K_HTTP_OK;
+	}
 
 	account = mse->account;
 	gc = exchange_account_get_global_catalog (account);
 	if (!gc) {
 		g_warning ("No GC: could not unmangle meeting request");
+		e2k_results_free (results, nresults);
 		return E2K_HTTP_OK;
 	}
 
@@ -1952,6 +1965,7 @@ unmangle_delegated_meeting_request (MailStubExchange *mse, E2kOperation *op,
 		&entry);
 	if (gcstatus != E2K_GLOBAL_CATALOG_OK) {
 		g_warning ("GC lookup failed: could not unmangle meeting request");
+		e2k_results_free (results, nresults);
 		return E2K_HTTP_OK;
 	}
 
@@ -1975,6 +1989,7 @@ unmangle_delegated_meeting_request (MailStubExchange *mse, E2kOperation *op,
 	}
 	e2k_global_catalog_entry_free (gc, entry);
 
+	e2k_results_free (results, nresults);
 	return E2K_HTTP_OK;
 }
 
@@ -2710,7 +2725,6 @@ unsubscribe_folder (MailStub *stub, const char *folder_name)
 		g_free (path);
 		return;
 	}
-	g_free (path);
 	g_object_ref (folder);
 
 	result = exchange_account_remove_favorite (mse->account, folder);
@@ -2724,16 +2738,19 @@ unsubscribe_folder (MailStub *stub, const char *folder_name)
 	case EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED:
 		mail_stub_return_error (stub, _("Permission denied"));
 		g_object_unref (folder);
+		g_free (path);
 		return;
 
 	default:
 		mail_stub_return_error (stub, _("Generic error"));
 		g_object_unref (folder);
+		g_free (path);
 		return;
 
 	}
 
 	g_object_unref (folder);
+	g_free (path);
 	mail_stub_return_ok (stub);
 }
 
@@ -2862,10 +2879,17 @@ static void
 folder_update_linestatus (gpointer key, gpointer value, gpointer data)
 {
 	MailStubExchangeFolder *mfld = (MailStubExchangeFolder *) value;
+	MailStub *stub = MAIL_STUB (mfld->mse);
 	gint linestatus = GPOINTER_TO_INT (data);
+	guint32 readonly;
 	
 	if (linestatus == ONLINE_MODE) {
 		get_folder_online (mfld, TRUE);
+		readonly = (mfld->access & (MAPI_ACCESS_MODIFY | MAPI_ACCESS_CREATE_CONTENTS)) ? 0 : 1;
+		mail_stub_return_data (stub, CAMEL_STUB_RETVAL_FOLDER_SET_READONLY,
+				       CAMEL_STUB_ARG_FOLDER, mfld->name,
+				       CAMEL_STUB_ARG_UINT32, readonly,
+				       CAMEL_STUB_ARG_END);
 	}
 	else {
 		/* FIXME: need any undo for offline */ ;
